<?php

namespace App\Models\Enums;

enum ObjectAction: string
{
    case UPLOADED = "uploaded";

    case UNDELETED = "undeleted";

    case SUBMITTED = "submitted";

    case STARTED = "started";

    case REVIEWED = "reviewed";

    case RESTARTED = "restarted";

    case RESPOND = "respond";

    case PAUSED = "paused";

    case OPENED = "opened";

    case MOVED = "moved";

    case ISSUED = "issued";

    case FINISHED = "finished";

    case CREATED = "created";

    case EDITED = "edited";

    case DONE = "done";

    case DELAY = "delay"; //延迟

    case DELETED = "deleted";

    case DOWNLOAD= "download";

    case CONFIRMED = "confirmed";

    case CLOSED = "closed";

    case CHANGED = "changed";

    case CANCELED = "canceled";

    case ASSIGNED = "assigned";

    case DELETED_FILE = "deletedFile";

    case EDITED_FILE = "editedFile";

    case HIDDEN = "hidden";

    case COMMENTED = "commented";

    case ACTIVATED = "activated";

    case BLOCKED = "blocked";

    case LINKED_TO_PLAN = "linkedToPlan";

    case CHANGED_STATUS = "change status";

    case VERIFIED = "verified";

    case SUSPENDED = "suspended";

    case LINK_CHILDREN_TASK = "linkChildrenTask";

    case UNLINK_CHILDREN_TASK = "unlinkChildrenTask";

    case BATCH_CREATE_TASK = "batchCreateTask";

    case DELETE_CHILDREN_TASK = "deleteChildrenTask";

    case WAITED ="waited";

    case APPROVAL_REQUEST= 'approvalRequest';

    case APPROVAL_APPROVED = 'approvalApproved';

    case APPROVAL_REJECTED = 'approvalRejected';

    case APPROVED_TO_NEXT_NODE ='approvedToNextStep';

    case APPROVED_CANCELED ='approvedCanceled';

    case APPROVED_WAIT_OTHERS = "approvedWaitOthers";

    public function isApproval()
    {
        return in_array($this->value, [
            self::APPROVAL_REQUEST->value,
            self::APPROVAL_APPROVED->value,
            self::APPROVAL_REJECTED->value,
            self::APPROVED_TO_NEXT_NODE->value,
            self::APPROVED_WAIT_OTHERS->value,
        ]);
    }

    public static function messageNotificationItems()
    {
        $approval = [
            self::APPROVAL_REQUEST,
            self::APPROVAL_APPROVED,
            self::APPROVAL_REJECTED,
            self::APPROVED_TO_NEXT_NODE,
            self::APPROVED_WAIT_OTHERS,
        ];

        return [
            ActionObjectType::REQUIREMENT->value => [
                self::CREATED,
                self::EDITED,
                self::STARTED,
                self::CHANGED,
                self::CLOSED,
                ...$approval,
            ],
            ActionObjectType::TASK->value => [
                ObjectAction::STARTED,
                ObjectAction::PAUSED,
                ObjectAction::CLOSED,
                ObjectAction::DONE,
                ObjectAction::CANCELED,
                ObjectAction::EDITED,
                ...$approval,
            ],
            ActionObjectType::CONTAINER->value => [
                ObjectAction::CREATED,
                ObjectAction::EDITED,
                ...$approval,
            ],
            ActionObjectType::PROJECT->value => [
                ...$approval,
            ]
        ];
    }
}
